# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from datetime import datetime
from django.http import HttpResponse

from django.conf import settings
from oauth_backend.models import Consumer, Token
from piston.authentication import OAuthAuthentication
from piston.oauth import OAuthError as PistonOAuthError

from identityprovider.models import Account, AccountPassword, EmailAddress
from identityprovider.models.api import APIUser
from identityprovider.models.const import EmailStatus
from identityprovider.utils import validate_launchpad_password


class LaunchpadBackend(object):

    def authenticate(self, username=None, password=None,
                     encrypted_password=None):
        try:
            email = EmailAddress.objects.get(email__iexact=username)
        except EmailAddress.DoesNotExist:
            return None

        if email.status not in (EmailStatus.PREFERRED,
                                EmailStatus.VALIDATED,
                                EmailStatus.NEW):
            return None
        account = email.account

        # only allow authentication if account is active
        if not account.is_active:
            return None

        try:
            expected_encrypted = account.accountpassword.password
        except AccountPassword.DoesNotExist:
            # password lost somehow, user should reset it
            return None

        def ok():
            account.last_login = datetime.now()
            return account

        if password is not None:
            if validate_launchpad_password(password, expected_encrypted):
                return ok()
        elif encrypted_password is not None:
            if encrypted_password == expected_encrypted:
                return ok()

    def get_user(self, user_id):
        """ Returns a Launchpad Account object instead of Django's built-in
        User object. """
        try:
            return Account.objects.get(pk=user_id)
        except Account.DoesNotExist:
            return None


def basic_authenticate(username, password):
    """Function used by lazr.authentication middleware"""
    api_user = APIUser.authenticate(username, password)
    if api_user is not None:
        return api_user
    user = LaunchpadBackend().authenticate(username, password)
    return user


class SSOOAuthAuthentication(OAuthAuthentication):
    def is_authenticated(self, request):
        if not self.is_valid_request(request):
            return False
        try:
            consumer, token, parameters = self.validate_token(request)
        except PistonOAuthError:
            return False
        if not (consumer and token):
            return False
        # only allow authentication if account is active
        account = Account.objects.get(openid_identifier=consumer.key)
        if not account.is_active:
            return False
        request.user = account
        request.throttle_extra = token.consumer.id
        return True

    def challenge(self):
        resp = HttpResponse("Authorization Required")
        resp['WWW-Authenticate'] = 'OAuth realm="%s"' % self.realm
        resp.status_code = 401
        return resp

def oauth_authenticate(oauth_consumer, oauth_token, parameters):
    """Currently only checks that given consumer and token are in database"""
    try:
        consumer = Consumer.objects.get(
            user__username=oauth_consumer.key)
        token = Token.objects.get(token=oauth_token.key)

    except (Token.DoesNotExist, Consumer.DoesNotExist):
        return None
    else:
        # only allow authentication if account is active
        account = Account.objects.get(openid_identifier=consumer.key)
        if not account.is_active:
            return None

        if token.consumer.key == oauth_consumer.key:
            return account
        else:
            return None


def internal_authorize(user):
    if user.is_authenticated() and user.person is not None:
        for team in getattr(settings, 'SSO_VERSIONS_TEAMS', []):
            if user.person.in_team(team):
                return True
    return False
