/* Copyright (c) 2009, Canonical Ltd. All rights reserved. */

YUI.add('lazr.passwordmeter', function(Y){

/**
 * A password strength meter widget.
 *
 *
 * Given the id of a password field and the name of a function that will check
 * the strength of the input password this meter will display on each character
 * entered the strength value computed by the function provided.
 *
 * @module lazr.passwordmeter
 * @namespace lazr
 */
Y.namespace('lazr');

/**
 * @class PasswordMeter
 * @extends Widget
 * @constructor
 */
var PasswordMeter = function(){
	PasswordMeter.superclass.constructor.apply(this, arguments);
};

PasswordMeter.NAME = 'passwordmeter';

/**
 * The HTML_PARSER static constant is used by the Widget base class to
 * populate the configuration for the PasswordMeter instance from markup
 * already on the page.
 *
 * @property PasswordMeter.HTML_PARSER
 * @type Object
 * @static
 */
PasswordMeter.HTML_PARSER = {
};

/**
 * This function will be used to determine the password's strength. It only
 * gives a 100% strength if the password has a combination of lower case, upper
 * case, numeric and symbolic characters with at least a length of 8.
 * The function can be replaed by the user of the widget by simply supplying
 * their own function in the "func" attribute.
 * The function returns an int between 0 and 4, where 0 is very weak and 4 is
 * very secure.
 */
var default_password_strength_function = function(password) {
    var color = '#8b0000';
    var text = '';
    if (password) {
        var len = password.length;
        var hasLower = /[a-z]/.test(password) ? 1 : 0;
        var hasUpper = /[A-Z]/.test(password) ? 1 : 0;
        var hasNumber = /\d/.test(password) ? 1 : 0;
        var hasSymbol = /[\~\`\!\@\#\$\%\^\&\*\(\)\_\-\+\=\{\}\[\]\:\;\"\'\<\>\?\|\\\,\.\/]/.test(password) ? 1 : 0;
        if (len < 8) {
            return 0;
        }
        else {
            var points = hasLower + hasUpper + hasNumber + hasSymbol - 1;
            return points >= 1 ? points : 1;
        }
    }
};

PasswordMeter.ATTRS = {
	/**
	 * Current color for the password strength
	 *
	 * @attribute color
	 * @type String
	 * @default #8b0000
	 */
	color: {
		value: '#8b0000'
	},

	/**
	 * Current description of password strength
	 *
	 * @attribute text
	 * @type String
	 * @default ""
	 */
	text: {
		value: ""
	},

	/**
	 * Password field we will be monitoring
	 *
	 * @attribute input
	 * @type Node
	 * @default null
	 */
	input: {
		value: null,
		setter: function(v){
			return Y.one(v);
		}
	},

	/**
	 * Javascript function that will calculate the password's strength
	 * and strength description
	 * Should return an object literal in the form
	 * {"color": <color of the text displaying the strength>,
	 *  "text": <textual description of the strength>}
	 *
	 * @attribute func
	 * @type function
	 * @default a standard password strength function
	 */
	func: {
		value: default_password_strength_function,
		validator: function(val) {
			return Y.Lang.isFunction(val);
		}
	},

	/**
	 * Password strength messages.
	 * These messages can be customized to provide i18n.
	 * There should be 5 messages in total, from very weak to very strong.
	 *
	 * @attribute messages
	 * @type Array
	 */
	messages: {
		value: ['Strength: too short', 'Strength: weak', 'Strength: fair',
				'Strength: good', 'Strength: strong']
	},

	/**
	 * Password strength colors.
	 * These colors can be customized to match the look and feel of your site.
	 * There should be 5 colors in total, from very weak to very strong.
	 *
	 * @attribute colors
	 * @type Array
	 */
	colors: {
		value: ['#8b0000', '#f99300', '#1e6400', '#1e6400', '#1e6400']
	}
}; //end ATTRS
Y.extend(PasswordMeter, Y.Widget, {
	/**
	 * Initialize the widget.
	 *
	 * @method initializer
	 * @protected
	 */
	initializer: function(cfg){
	},

	/**
	 * Destroy the widget.
	 *
	 * @method dest
	 * @protected
	 */
	destructor: function(){
	},

	/**
	 * Update the DOM structure and edit CSS classes.
	 *
	 * @method renderUI
	 * @protected
	 */
	renderUI: function(){
		this.get("contentBox").setStyle('width', this.get('input').getStyle('width'));
		this.get("contentBox").setStyle('height', this.get('input').getStyle('height'));
		this.calculateStrength();
		this.updateDisplay();
	},

	/**
	 * Set the event handlers for the input element.
	 *
	 * @method bindUI
	 * @protected
	 */
	bindUI: function(){
		var input = this.get('input');
		input.on('keyup', this._onKeyup, this);
	},

	/**
	 * Synchronize the DOM with our current attribute state
	 *
	 * @method syncUI
	 * @protected
	 */
	syncUI: function(){
		this.updateDisplay();
	},

	/**
	 * Capture each password character
	 *
	 * @method _onKeyup
	 * @protected
	 * @param e {Event.Custom} The event object.
	 */
	_onKeyup: function(e){
		this.calculateStrength();
		this.syncUI();
	},

	updateDisplay: function() {
		this.get('contentBox').set('innerHTML', this.get('text'));
		this.get('contentBox').setStyle("color", this.get('color'));
	},

	calculateStrength: function() {
		var input = this.get('input');
		var password = input.get('value');
		if (password) {
			var result = this.get('func')(password);
			this.set('color', this.get('colors')[result]);
			this.set('text', this.get('messages')[result]);
		}
		else {
			this.set('text', '');
		}
	}
}); //end extend

Y.PasswordMeter = PasswordMeter;
}, "0.1", {
    "skinnable": true,
    "requires": ["widget", "lazr.base"]
});
