import django

from configglue.pyschema import (
    BoolConfigOption,
    ConfigSection,
    DictConfigOption,
    IntConfigOption,
    LinesConfigOption,
    StringConfigOption,
    TupleConfigOption,
)
from django_configglue.schema import schemas


if django.VERSION[:2] == (1, 3):
    DjangoSchema = schemas.get('1.3')
else:
    DjangoSchema = schemas.get('1.1.4')


class UpperCaseDictConfigOption(DictConfigOption):
    """ A DictConfigOption with all upper-case keys. """
    def parse(self, section, parser=None, raw=False):
        parsed = super(UpperCaseDictConfigOption, self).parse(
            section, parser, raw)
        result = {}
        for k, v in parsed.items():
            result[k.upper()] = v
        return result


class SSOSchema(DjangoSchema):
    # default
    branded_template_dir = StringConfigOption()
    db_connections = LinesConfigOption(item=UpperCaseDictConfigOption(
        spec={'database_id': StringConfigOption(),
              'database_host': StringConfigOption(),
              'database_name': StringConfigOption(),
              'database_user': StringConfigOption(),
              'database_password': StringConfigOption(),
              'database_port': StringConfigOption()},
        strict=True))
    db_statement_timeout_millis = IntConfigOption(
        default=0, help='0 disables timeout')
    extra_pythonpath = LinesConfigOption(item=StringConfigOption())
    oops_dir = StringConfigOption(help='Absolute path to the directory oops'
        ' reports will be stored in')
    pgconnect_timeout = IntConfigOption(default=10)
    serve_static_media = BoolConfigOption()
    support_form_url = StringConfigOption()
    test_dsn = StringConfigOption()

    #i18n
    i18n = ConfigSection()
    i18n.supported_languages = LinesConfigOption(item=StringConfigOption(),
        help="List of currently supported languages")
    i18n.language_names = DictConfigOption(
        help="Mapping between language-code and name in that language")

    # api
    api = ConfigSection()
    api.api_enabled = BoolConfigOption()
    api.api_host = StringConfigOption()
    api.oauth_data_store = StringConfigOption()

    # branding
    branding = ConfigSection()
    branding.brand = StringConfigOption()

    # captcha
    captcha = ConfigSection()
    captcha.captcha_use_proxy = BoolConfigOption()
    captcha.captcha_proxies = DictConfigOption()
    captcha.captcha_api_url = StringConfigOption()
    captcha.captcha_api_url_secure = StringConfigOption()
    captcha.captcha_image_url_pattern = StringConfigOption(raw=True)
    captcha.captcha_verify_url = StringConfigOption()
    captcha.captcha_public_key = StringConfigOption()
    captcha.captcha_private_key = StringConfigOption()
    captcha.disable_captcha_verification = BoolConfigOption()

    # debug
    debug = ConfigSection()
    debug.openid_debug = BoolConfigOption()
    debug.testing = BoolConfigOption()

    # email
    email = ConfigSection()
    email.noreply_from_address = StringConfigOption()
    email.feedback_to_address = StringConfigOption()
    email.enable_token_debug = BoolConfigOption()

    # openid
    openid = ConfigSection()
    openid.pre_authorization_validity = IntConfigOption()
    openid.openid_preauthorization_acl = LinesConfigOption(
        item=TupleConfigOption(length=2))

    # preflight
    preflight = ConfigSection()
    preflight.preflight_base_template = StringConfigOption()

    # readonly
    readonly = ConfigSection()
    readonly.app_servers = LinesConfigOption(item=UpperCaseDictConfigOption(
        spec={'server_id': StringConfigOption(),
              'host': StringConfigOption(),
              'port': StringConfigOption(),
              'scheme': StringConfigOption(default='http'),
              'virtual_host': StringConfigOption(),
              },
        strict=True))
    readonly.readonly_secret = StringConfigOption(raw=True)
    readonly.dbfailover_attempts = IntConfigOption()
    readonly.dbfailover_flag_dir = StringConfigOption()
    readonly.dbrecover_interval = IntConfigOption()
    readonly.dbrecover_attempts = IntConfigOption()
    readonly.dbrecover_multiplier = IntConfigOption()
    readonly.read_only_mode = BoolConfigOption(default=False)

    # sso
    sso = ConfigSection()
    sso.sso_media_root = StringConfigOption()
    sso.sso_restrict_rp = BoolConfigOption()
    sso.sso_root_url = StringConfigOption()
    sso.sso_provider_url = StringConfigOption()
    sso.sso_versions_teams = LinesConfigOption(
        item=StringConfigOption(),
        help="List of teams able to access /+versions page")
    sso.sso_account_update_url = StringConfigOption(
        null=True,
        help="URL to POST when account data has changed")
    sso.logging_config_file = StringConfigOption(
        default="config/logging.conf",
        help="Path to logging configuration file")
    sso.embedded_trust_root = StringConfigOption()

    # yubico
    yubico = ConfigSection()
    yubico.yubico_client_id = StringConfigOption(fatal=True)
    yubico.yubico_api_url = StringConfigOption(
        default=('http://api.yubico.com/wsapi/2.0/verify?id={client_id}&'
                 'otp={otp}&nonce={nonce}'))

    profile = ConfigSection()
    profile.profile_filename = StringConfigOption(
        default="sso.profile",
        help="Filename of the file created by the profile middleware")
    profile.profile_pattern = StringConfigOption(
        default="identityprovider\..*",
        help="Regex pattern for limiting functions to be considered for profiling")

    devserver = ConfigSection()
    devserver.devserver_ignored_prefixes = LinesConfigOption(
        item=StringConfigOption(),
        help="List of url prefixes which will be ignored")
    devserver.devserver_modules = LinesConfigOption(
        item=StringConfigOption(),
        help="List of devserver modules to enable")
    devserver.devserver_default_addr = StringConfigOption(default="127.0.0.1")
    devserver.devserver_default_port = StringConfigOption(default="8000")
    devserver.devserver_truncate_sql = BoolConfigOption(default=True)
    devserver.devserver_ajax_content_length = IntConfigOption(default=300)
    devserver.devserver_auto_profile = BoolConfigOption(default=False)
