# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os
import fnmatch
import unittest
import logging

from openid import oidutil
from django.conf import settings
from django.test import _doctest as doctest
from django.test.testcases import OutputChecker, DocTestRunner
from identityprovider import decorators


def setup_package():
    decorators.disable_cookie_check = True
    # Disable writing OpenID messaged to stderr
    oidutil.log = logging.debug

def teardown_package():
    settings.DATABASE_NAME = "test_{0}".format(settings.DATABASE_NAME)

def suite():
    test_suite = unittest.TestSuite()
    app_dir = _get_application_directory()
    app_name = os.path.basename(app_dir)

    _add_unittest_modules(test_suite, app_name, app_dir)
    _add_doctest_modules(test_suite, app_name, app_dir)

    return test_suite


def _get_application_directory():
    return os.path.realpath(os.path.join(os.path.dirname(__file__), '..'))


def _add_doctest_modules(test_suite, app_name, app_dir):
    for root, dirs, files in os.walk(app_dir):
        root = _normalize_directory_name(root, app_dir)
        if _should_skip_directory(root):
            continue
        for test_file in files:
            if not _valid_module_name(test_file):
                continue
            test_file = test_file.replace('.py', '')
            module_name = _doctest_module_name(test_file, root, app_name)
            doctest_test = _create_doctest_test(module_name, test_file)
            if doctest_test:
                test_suite.addTest(doctest_test)


def _valid_module_name(test_file):
    if fnmatch.fnmatch(test_file, '*.py'):
        if test_file[0] not in ['_', '.', '~']:
            return True
    return False


def _doctest_module_name(test_file, root, app_name):
    module_path = [app_name]
    if root:
        module_path.extend(root.split('/'))
    module_path.append(test_file)
    module_name = '.'.join(module_path)
    return module_name


def _create_doctest_test(module_name, test_file):
    module = __import__(module_name, fromlist=[test_file])
    try:
        return doctest.DocTestSuite(module, checker=OutputChecker(),
                                    runner=DocTestRunner)
    except ValueError:
        # If there's no test in module just continue
        pass


def _normalize_directory_name(root, app_dir):
    return root.replace(app_dir, '').lstrip('/')


def _should_skip_directory(root):
    return (root.startswith('tests') or
            root.startswith('models') or
            root.startswith('.'))


def _add_unittest_modules(test_suite, app_name, app_dir):
    for fn in os.listdir(os.path.join(app_dir, 'tests')):
        if fnmatch.fnmatch(fn, 'test_*.py'):
            test_file_name = fn.replace('.py', '')
            test_module_name = '%s.tests.%s' % (app_name, test_file_name)
            test_module = __import__(test_module_name,
                                     fromlist=[test_file_name])
            test = unittest.defaultTestLoader.loadTestsFromModule(test_module)
            test_suite.addTest(test)
