import base64

from django.utils import simplejson
from mock import patch

from identityprovider.tests.utils import SQLCachedTestCase
from identityprovider.models import (
    Account,
    APIUser,
    AuthToken,
    AuthTokenFactory,
    EmailAddress,
)
from identityprovider.models.const import (
    AccountStatus,
    EmailStatus,
    LoginTokenType,
)
from identityprovider.api10.handlers import (
    AuthenticationHandler,
    CanNotResetPasswordError,
    RegistrationHandler,
)


class MockRequest(object):
    def __init__(self, data=None, user=None):
        self.user = user
        if data is None:
            data = {}
        self.data = data
        self.environ = {'REMOTE_ADDR': '127.0.0.1'}

class RegistrationHandlerTestCase(SQLCachedTestCase):

    fixtures = ["test"]
    pgsql_functions = ["generate_openid_identifier"]

    def setUp(self):
        self.registration = RegistrationHandler()

    def test_authtoken_without_requester_is_handled_properly(self):
        authtoken = AuthTokenFactory().new(None, None, "test@example.com",
                                           LoginTokenType.PASSWORDRECOVERY,
                                           None)
        request = MockRequest(data={'email': "test@example.com",
            'token': authtoken.token, 'new_password': "pass"})

        response = self.registration.set_new_password(request)

        self.assertEquals(response['status'], "error")

    def test_reset_token_when_email_is_invalid(self):
        request = MockRequest(data={'email': "non-existing@example.com"})

        self.assertRaises(CanNotResetPasswordError,
                          self.registration.request_password_reset_token,
                          request)

    def test_reset_token_when_account_is_disabled(self):
        account = Account.objects.get_by_email("mark@example.com")
        account.status = AccountStatus.SUSPENDED
        account.save()
        request = MockRequest(data={'email': "mark@example.com"})

        self.assertRaises(CanNotResetPasswordError,
                          self.registration.request_password_reset_token,
                          request)

    def test_reset_password_when_not_existing_token_is_passed(self):
        request = MockRequest(data={'email': "mark@example.com",
            'token': "not-existing-token", 'new_password': "password"})

        self.assertRaises(AuthToken.DoesNotExist,
                          self.registration.set_new_password, request)

    def test_register_email_error_is_in_list(self):
        request = MockRequest(data={
            'email': "register@example.com",
            'password': "blogdf3Daa",
            'captcha_solution': "solution",
            'captcha_id': "id",
        })
        EmailAddress.objects.create(email="register@example.com",
            status=EmailStatus.NEW)

        registration = RegistrationHandler()
        r = registration.register(request)

        self.assertEquals(r['errors'], {'email': ["Email already registered"]})

    def test_register_without_displayname(self):
        request = MockRequest(data={
            'email': "test@example.com",
            'password': "MySecretPassword1",
            'captcha_solution': "foobar",
            'captcha_id': "id",
        })
        response = self.registration.register(request)
        self.assertEqual(response['status'], 'ok')

        account = Account.objects.get_by_email('test@example.com')
        self.assertEqual(account.displayname, '')

    def test_register_with_displayname(self):
        request = MockRequest(data={
            'email': "test@example.com",
            'password': "MySecretPassword1",
            'captcha_solution': "foobar",
            'captcha_id': "id",
            'displayname': 'Test User',
        })
        response = self.registration.register(request)
        self.assertEqual(response['status'], 'ok')

        account = Account.objects.get_by_email('test@example.com')
        self.assertEqual(account.displayname, 'Test User')

    @patch('identityprovider.models.authtoken.AuthToken.sendNewUserEmail')
    def test_register_without_platform(self, mock_sendNewUserEmail):
        request = MockRequest(data={
            'email': 'test@example.com',
            'password': 'MySecretPassword1',
            'captcha_solution': 'foobar',
            'captcha_id': 'id',
        })
        response = self.registration.register(request)
        self.assertEqual(response['status'], 'ok')

        mock_sendNewUserEmail.assert_called_with(template='api-newuser.txt')

    @patch('identityprovider.models.authtoken.AuthToken.sendNewUserEmail')
    def test_register_with_platform(self, mock_sendNewUserEmail):
        request = MockRequest(data={
            'email': 'test@example.com',
            'password': 'MySecretPassword1',
            'captcha_solution': 'foobar',
            'captcha_id': 'id',
            'platform': 'mobile',
        })
        response = self.registration.register(request)
        self.assertEqual(response['status'], 'ok')

        mock_sendNewUserEmail.assert_called_with(template='newuser.txt')

    @patch('identityprovider.models.authtoken.AuthToken.sendNewUserEmail')
    def test_register_with_invalid_platform(self, mock_sendNewUserEmail):
        request = MockRequest(data={
            'email': 'test@example.com',
            'password': 'MySecretPassword1',
            'captcha_solution': 'foobar',
            'captcha_id': 'id',
            'platform': 'foo',
        })
        response = self.registration.register(request)
        self.assertEqual(response['status'], 'error')
        self.assertTrue('platform' in response['errors'])

    @patch('identityprovider.models.authtoken.AuthTokenFactory.'
        'new_api_email_validation_token')
    def test_register_without_redirection_url(self,
            mock_new_api_email_validation_token):
        request = MockRequest(data={
            'email': 'test@example.com',
            'password': 'MySecretPassword1',
            'captcha_solution': 'foobar',
            'captcha_id': 'id',
            'platform': 'desktop',
        })
        response = self.registration.register(request)
        self.assertEqual(response['status'], 'ok')

        email = EmailAddress.objects.get(email='test@example.com')
        mock_new_api_email_validation_token.assert_called_with(
            email.account, 'test@example.com', None)

    @patch('identityprovider.models.authtoken.AuthTokenFactory.'
        'new_api_email_validation_token')
    def test_register_from_desktop_with_redirection_url(self,
            mock_new_api_email_validation_token):
        request = MockRequest(data={
            'email': 'test@example.com',
            'password': 'MySecretPassword1',
            'captcha_solution': 'foobar',
            'captcha_id': 'id',
            'platform': 'desktop',
            'validate_redirect_to': 'http://foo',
        })
        response = self.registration.register(request)
        self.assertEqual(response['status'], 'ok')

        email = EmailAddress.objects.get(email='test@example.com')
        mock_new_api_email_validation_token.assert_called_with(
            email.account, 'test@example.com', 'http://foo')

    @patch('identityprovider.api10.handlers.encrypt_launchpad_password')
    @patch('identityprovider.models.authtoken.AuthTokenFactory.new')
    def test_register_from_mobile_with_redirection_url(self, mock_new,
            mock_encrypt_launchpad_password):

        mock_encrypt_launchpad_password.return_value = 'the_password'

        request = MockRequest(data={
            'email': 'test@example.com',
            'password': 'MySecretPassword1',
            'captcha_solution': 'foobar',
            'captcha_id': 'id',
            'platform': 'mobile',
            'validate_redirect_to': 'http://foo',
        })
        response = self.registration.register(request)
        self.assertEqual(response['status'], 'ok')

        mock_new.assert_called_with(None, None, 'test@example.com',
            LoginTokenType.NEWPERSONLESSACCOUNT, 'http://foo',
            displayname='', password='the_password')

    @patch('identityprovider.api10.handlers.encrypt_launchpad_password')
    @patch('identityprovider.models.authtoken.AuthTokenFactory.new')
    def test_register_from_mobile_without_redirection_url(self, mock_new,
            mock_encrypt_launchpad_password):

        mock_encrypt_launchpad_password.return_value = 'the_password'

        request = MockRequest(data={
            'email': 'test@example.com',
            'password': 'MySecretPassword1',
            'captcha_solution': 'foobar',
            'captcha_id': 'id',
            'platform': 'mobile',
        })
        response = self.registration.register(request)
        self.assertEqual(response['status'], 'ok')

        mock_new.assert_called_with(None, None, 'test@example.com',
            LoginTokenType.NEWPERSONLESSACCOUNT, None,
            displayname='', password='the_password')


class AuthenticationTestCase(SQLCachedTestCase):

    fixtures = ["test"]

    def setUp(self):
        self.authentication = AuthenticationHandler()

    def test_account_by_openid_with_valid_openid(self):
        request = MockRequest(user=APIUser(), data={'openid': "mark_oid"})

        account = self.authentication.account_by_openid(request)

        self.assertEquals(account['openid_identifier'], "mark_oid")

    def test_account_by_openid_with_invalid_openid(self):
        request = MockRequest(user=APIUser(), data={'openid': "bad-openid"})

        account = self.authentication.account_by_openid(request)

        self.assertTrue(account is None)

    def test_authenticate_unquoted_token(self):
        expected_keys = ['consumer_key', 'consumer_secret', 'name', 'token',
            'token_secret']

        extra = {'HTTP_AUTHORIZATION': 'basic ' + base64.b64encode('mark@example.com:test')}
        response = self.client.get('/api/1.0/authentications', data={
            'ws.op': 'authenticate', 'token_name': 'some-token'}, **extra)

        content = simplejson.loads(response.content)
        self.assertEqual(set(content.keys()), set(expected_keys))
        self.assertEqual(content['name'], 'some-token')

    def test_authenticate_quoted_token(self):
        expected_keys = ['consumer_key', 'consumer_secret', 'name', 'token',
            'token_secret']

        extra = {'HTTP_AUTHORIZATION': 'basic ' + base64.b64encode('mark@example.com:test')}
        response = self.client.get('/api/1.0/authentications', data={
            'ws.op': 'authenticate', 'token_name': '"some-token"'}, **extra)

        content = simplejson.loads(response.content)
        self.assertEqual(set(content.keys()), set(expected_keys))
        self.assertEqual(content['name'], 'some-token')

