# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import unittest
import urllib2

from mock import patch_object

from cStringIO import StringIO
from django.conf import settings

from identityprovider.models.captcha import (
    Captcha,
    CaptchaResponse,
    NewCaptchaError,
    VerifyCaptchaError,
)


class CaptchaResponseTestCase(unittest.TestCase):

    def test_no_data(self):
        r = CaptchaResponse(200, None)
        self.assertEqual(r.data(), None)

    def test_data_from_response(self):
        resp = StringIO('this is the response')
        r = CaptchaResponse(200, resp)
        self.assertEqual(r._data, None)
        self.assertEqual(r.data(), 'this is the response')

    def test_data_exists(self):
        r = CaptchaResponse(200, None)
        r._data = 'this is the data'
        self.assertEqual(r.data(), 'this is the data')


class CaptchaTestCase(unittest.TestCase):

    @patch_object(Captcha, '_open')
    def test_new_captcha_error(self, mock_open):
        self.assertRaises(NewCaptchaError, Captcha.new)

    @patch_object(Captcha, '_open')
    def test_new_captcha_no_challenge(self, mock_open):
        try:
            Captcha.new()
        except NewCaptchaError, e:
            self.assertEqual(e.dummy.captcha_id, None)
            self.assertEqual(e.dummy.image_url, None)

    def test_captcha_open(self):
        class MockOpener(object):
            def open(self, request):
                raise urllib2.URLError((-1, 'error'))
        old_opener = Captcha.opener
        Captcha.opener = MockOpener()

        self.assertRaises(urllib2.URLError, Captcha.new)

        Captcha.opener = old_opener


class CaptchaVerifyTestCase(unittest.TestCase):

    def setUp(self):
        self.old_DISABLE_CAPTCHA_VERIFICATION = settings.DISABLE_CAPTCHA_VERIFICATION
        settings.DISABLE_CAPTCHA_VERIFICATION = False

    def tearDown(self):
        settings.DISABLE_CAPTCHA_VERIFICATION = self.old_DISABLE_CAPTCHA_VERIFICATION

    @patch_object(Captcha, '_open')
    def test_verify_already_verified(self, mock_open):
        c = Captcha(None)
        c._verified = True
        r = c.verify(None, None)
        self.assertTrue(r)

    @patch_object(Captcha, '_open')
    def test_verify_no_verification(self, mock_open):
        settings.DISABLE_CAPTCHA_VERIFICATION = True

        c = Captcha(None)
        r = c.verify(None, None)
        self.assertTrue(r)

    @patch_object(Captcha, '_open')
    def test_verify_response_ok(self, mock_open):
        @classmethod
        def mock_open(cls, request):
            r = CaptchaResponse(200, StringIO('true\nok'))
            return r
        old_open = Captcha._open
        Captcha._open = mock_open

        c = Captcha.new()
        r = c.verify(None, "localhost")

        self.assertTrue(r)
        self.assertEqual(c.message, 'ok')

        Captcha._open = old_open

    @patch_object(Captcha, '_open')
    def test_verify_no_captcha_id(self, mock_open):
        c = Captcha(None)
        r = c.verify(None, 'localhost')

        self.assertFalse(r)
        self.assertEqual(c.message, 'no-challenge')

    @patch_object(Captcha, '_open')
    def test_verify_error(self, mock_open):
        c = Captcha(None)
        c.captcha_id = 'challenge-id'

        self.assertRaises(VerifyCaptchaError, c.verify, None, 'localhost')
