# encoding: utf-8
# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import logging
import re
import urllib2
from urlparse import urlsplit

from mock import patch_object, patch

from django.conf import settings
from django.core import mail
from django.core.urlresolvers import Resolver404
from django.http import QueryDict
from django.test.client import Client
from django.utils.http import urlquote

from openid.message import IDENTIFIER_SELECT

from identityprovider import decorators, signed
from identityprovider.models import AUTHTOKEN_LENGTH, authtoken as at
from identityprovider.models import EmailAddress, Person, OpenIDRPConfig
from identityprovider.models.const import EmailStatus
from identityprovider.views import ui, server
from identityprovider.models import Account, AuthToken
from identityprovider.models.authtoken import create_token
from identityprovider.models.captcha import Captcha
from identityprovider.models.const import AccountStatus, LoginTokenType
from identityprovider.tests.utils import (
    BasicAccountTestCase,
    LPAccountTestCase, SQLCachedTestCase,
    MockHandler,
)
from identityprovider.utils import is_django_13


def _post_new_account(client, email='person@example.com', token=None):
    if token is None:
        url = '/+new_account'
    else:
        url = '/%s/+new_account' % token
    return client.post(url, {
        'displayname': 'Mark Shuttleworth',
        'email': email,
        'password': 'Testing123',
        'passwordconfirm': 'Testing123'})


class UIViewsHelperTestCase(SQLCachedTestCase):


    def test_is_safe_redirect_url_return_false(self):
        self.assertFalse(ui._is_safe_redirect_url('non-existing'))


    def test_is_safe_redirect_url_with_params(self):
        self.assertFalse(ui._is_safe_redirect_url('non-existing?q=some_value'))


    @patch('identityprovider.views.ui.resolve')
    def test_is_safe_redirect_url_404(self, mock_resolve):
        mock_resolve.side_effect = Resolver404()

        self.assertFalse(ui._is_safe_redirect_url('non-existing'))



class SpanishUIViewsTestCase(BasicAccountTestCase):


    def setUp(self):
        self.language_code = settings.LANGUAGE_CODE
        settings.LANGUAGE_CODE = 'es'


    def tearDown(self):
        settings.LANGUAGE_CODE = self.language_code


    def test_new_account_in_spanish(self):
        account = Account.objects.get_by_email('mark@example.com')
        account.status = AccountStatus.ACTIVE
        account.save()

        r = _post_new_account(self.client, email='mark@example.com')

        self.assertRedirects(r, '/+email-sent')
        self.assertEqual(len(mail.outbox), 1)



class UIViewsTestCase(BasicAccountTestCase):

    def setUp(self):
        logging.disable(logging.WARNING)
        self.disable_csrf()
        self._ENABLE_TOKEN_DEBUG = getattr(settings, 'ENABLE_TOKEN_DEBUG')
        settings.ENABLE_TOKEN_DEBUG = True

    def tearDown(self):
        settings.ENABLE_TOKEN_DEBUG = self._ENABLE_TOKEN_DEBUG
        self.reset_csrf()

    def _session_token(self):
        token_string = self.client.session['token_string']
        return AuthToken.objects.get(token=token_string)

    def _token_url(self, view, token_string=None, token_email=None):
        token_string = token_string or self.client.session['token_string']
        token_email = token_email or self.client.session['token_email']
        quoted_email = urlquote(token_email, safe='')
        return '/token/%s/%s/%s' % (token_string, view, quoted_email)

    def authenticate(self):
        self.client.login(username='mark@example.com', password='test')

    def test_login_with_unicode_email(self):
        email = u'mark@客example.com'
        password = 'test'
        Account.objects.create_account('Mark Shuttleworth', email, password)
        r = self.client.post(
            '/+login',
            {
                'email': email.encode('utf-8'),
                'password': password,
            })
        if is_django_13():
            self.assertRedirects(r, '/')
        else:
            self.assertFormError(r, 'form', 'email', 'Invalid email.')

    def test_login_with_unicode_password(self):
        r = self.client.post(
            '/+login',
            {
                'email': 'mark@example.com',
                'password': u'test客'.encode('utf-8'),
            })
        self.assertFormError(r, 'form', None, 'Password didn\'t match.')

    def test_login_with_next(self):
        r = self.client.post('/+login', {'email': 'mark@example.com',
                                         'password': 'test',
                                         'next': '/+edit'})
        self.assertRedirects(r, '/+edit')

    def test_login_when_account_inactive(self):
        account = Account.objects.get_by_email('mark@example.com')
        _password = account.accountpassword.password

        for status, _ in AccountStatus._get_choices():
            if status == AccountStatus.ACTIVE:
                # skip as this is already tested by test_login_with_next
                continue

            account.status = status
            account.save()
            # reset password in case previous status was suspended, which
            # resets the password
            account.accountpassword.password = _password
            account.accountpassword.save()

            r = self.client.post('/+login', {'email': 'mark@example.com',
                                             'password': 'test',
                                             'next': '/+edit'})

            self.assertFormError(r, 'form', None,
                                 'Your account has been deactivated')

    def test_login_without_next_with_token(self):
        token = 'a'*16
        r = self.client.post("/%s/+login" % token,
            {'email': 'mark@example.com', 'password': 'test'})
        self.assertEqual(r.status_code, 302)
        self.assertEqual(r['Location'], "http://testserver/%s/" % token)

    def test_login_without_next_nor_token(self):
        r = self.client.post('/+login', {'email': 'mark@example.com',
                                         'password': 'test'})
        self.assertRedirects(r, '/')

    def test_logout(self):
        self.authenticate()
        r = self.client.get('/+logout')
        self.assertTemplateUsed(r, 'ubuntu/registration/logout.html')

    def test_logout_preserve_token(self):
        self.authenticate()
        token = 'a' * 16
        # This strange dance with session is necessary to overcome way in which
        # test.Client returns session (recreating it on every Client.session
        # access)
        session = self.client.session
        session[token] = 'raw_orequest content'
        session.save()

        self.client.get('/%s/+logout' % token)
        self.assertEquals(self.client.session.get(token),
                          'raw_orequest content')

    def create_token(self, token_type, email=None, redirection_url=None,
            displayname=None, password=None):
        token = at.AuthToken.objects.create(
            token_type=token_type,
            token=at.create_unique_token_for_table(
                AUTHTOKEN_LENGTH, at.AuthToken, 'token'),
        )
        if email:
            token.email = email
        if redirection_url:
            token.redirection_url = redirection_url
        if displayname:
            token.displayname = displayname
        if password:
            token.password = password
        token.save()
        return token

    def test_claim_token_for_password_recovery_no_preferredemail(self):
        """ According to bug #524582 this should be handled gracefully by
        verifying the email address and resetting the password. """
        account = Account.objects.get_by_email('test@canonical.com')
        _preferred_email = account.preferredemail
        for email_obj in account.emailaddress_set.all():
            email_obj.status = EmailStatus.NEW
            email_obj.save()

        r = self.client.post('/+forgot_password',
                             {'email': 'test@canonical.com'})

        # reset password
        data = {'password': 'Password1', 'passwordconfirm': 'Password1'}
        r = self.client.post(self._token_url('+resetpassword'), data)
        self.assertRedirects(r, '/')

        for email_obj in account.emailaddress_set.all():
            email_obj.status = EmailStatus.VALIDATED
            email_obj.save()
        account.preferredemail = _preferred_email

    def test_enter_unexisting_token(self):
        r = self.client.post('/+enter_token', {
            'email': 'fake@example.com',
            'confirmation_code': '0'})
        self.assertContains(r, 'Unknown confirmation code')

    def test_claim_unexisting_token(self):
        r = self.client.get('/token/%s/' % 0, {'email': 'fake%40example.com'})
        self.assertEquals(r.status_code, 404)

    def test_claim_token_for_unexisting_token_type(self):
        token = self.create_token(9999, 'fake@example.com')
        r = self.client.get('/token/%s/' % token.token,
                            {'email': 'fake%40example.com'})
        self.assertEquals(r.status_code, 404)

    def test_claim_consumed_token(self):
        self.client.post('/+forgot_password',
                                 {'email': 'test@canonical.com'})
        url = self._token_url('+resetpassword')

        pwd = 'Password1'
        r = self.client.post(url, {'password': pwd, 'passwordconfirm': pwd})
        self.assertRedirects(r, '/')

        self.client.cookies.clear()

        pwd = 'Password2'
        r = self.client.post(url, {'password': pwd, 'passwordconfirm': pwd})
        self.assertRedirects(r, '/+bad-token')

    def test_token_form_validation(self):
        r = self.client.post('/+enter_token')
        self.assertContains(r, '<span class="error"')

    def test_config_email(self):
        self.authenticate()

        token = self.create_token(at.LoginTokenType.VALIDATEEMAIL,
            email='mark@example.com', redirection_url="/")
        url = '/token/%s/+newemail/%s' % (token.token, 'mark%40example.com')
        r = self.client.post(url, {'post': 'yes'})
        self.assertRedirects(r, token.redirection_url)

    def test_token_from_email_when_it_is_turned_on(self):
        r = self.get_debug_token_response(True)
        self.assertTemplateUsed(r, 'registration/token.html')

    def get_debug_token_response(self, debug_setting):
        self.create_token(at.LoginTokenType.VALIDATEEMAIL,
                                  email="mark@example.com")
        enable_token_debug = getattr(settings, 'ENABLE_TOKEN_DEBUG', None)
        try:
            settings.ENABLE_TOKEN_DEBUG = debug_setting
            r = self.client.get('/debug-token/mark@example.com')
        finally:
            settings.ENABLE_TOKEN_DEBUG = enable_token_debug
        return r

    def test_token_form_email_when_it_is_turned_off(self):
        r = self.get_debug_token_response(False)
        self.assertEquals(r.status_code, 404)

    def test_bad_token(self):
        r = self.client.get('/+bad-token')
        self.assertEquals(r.status_code, 200)

    def test_non_field_errors_is_not_in_html(self):
        r = self.client.post('/+enter_token', {
            'email': 'fake@example.com',
            'confirmation_code': '0'})
        # non field error section immediately follows a <p>
        self.assertNotContains(r, '<p><span class="error"')
        self.assertContains(r, '<span class="error"')

    def test_logout_to_confirm(self):
        r = self.client.get('/+logout-to-confirm')
        self.assertEquals(r.status_code, 200)

    def test_confirm_account_while_logged_in(self):
        token = self.create_token(at.LoginTokenType.NEWPERSONLESSACCOUNT,
                                  email='me@example.com')
        self.authenticate()
        r = self.client.get('/token/%s/' % token.token,
                            {'email': 'me@example.com'})
        location = None
        for item in r.items():
            if item[0] == 'Location':
                location = item[1]
                break
        location = location[len('http://testserver'):]
        r = self.client.get(location)
        self.assertRedirects(r,  '/+logout-to-confirm')

    def test_confirm_account_redirect_to_decide_token_error(self):
        # get a valid session
        token1 = 'a'*16
        r = _post_new_account(self.client, token=token1)
        self.assertRedirects(r, '/%s/+email-sent' % token1)

        # claim token
        r = self.client.post(self._token_url('+newaccount'),
            {'displayname': 'Person', 'password': 'P4ssw0rd',
             'passwordconfirm': 'P4ssw0rd'})
        self.assertRedirects(r, "/%s/+decide" % token1)

    def test_confirm_account_redirect_to_decide_token(self):
        # get a valid session
        token1 = create_token(16)
        r = _post_new_account(self.client, token=token1)
        self.assertRedirects(r, '/%s/+email-sent' % token1)

        # claim token
        r = self.client.post(self._token_url('+newaccount'),
            {'displayname': 'Person', 'password': 'P4ssw0rd',
             'passwordconfirm': 'P4ssw0rd'})
        self.assertRedirects(r, "/%s/+decide" % token1)

    def test_confirm_account_redirect_to_decide_with_rpconfig(self):
        token1 = create_token(16)
        r = _post_new_account(self.client, token=token1)
        self.assertRedirects(r, '/%s/+email-sent' % token1)

        request = {'openid.mode': 'checkid_setup',
                   'openid.trust_root': 'http://localhost/',
                   'openid.return_to': 'http://localhost/',
                   'openid.identity': IDENTIFIER_SELECT}
        openid_server = server._get_openid_server()
        orequest = openid_server.decodeRequest(request)
        session = self.client.session
        session[token1] = signed.dumps(orequest, settings.SECRET_KEY)
        session.save()

        # create rpconfig
        rpconfig = OpenIDRPConfig.objects.create(trust_root='http://localhost/')

        # claim token
        r = self.client.post(self._token_url('+newaccount'),
            {'displayname': 'Person', 'password': 'P4ssw0rd',
             'passwordconfirm': 'P4ssw0rd'})
        self.assertRedirects(r, "/%s/+decide" % token1)

        # verify account created
        account = Account.objects.get(pk=self.client.session['_auth_user_id'])
        self.assertEqual(account.creation_rationale,
                         rpconfig.creation_rationale)

    def test_confirm_account_without_redirection_url(self):
        token = self.create_token(at.LoginTokenType.NEWPERSONLESSACCOUNT,
            email='me@example.com', displayname='Me', password='password')
        self.assertEqual(token.redirection_url, None)

        # confirm account
        url = '/confirm-account/%s/%s' % (token.token, 'me%40example.com')
        r = self.client.post(url, {'post': 'yes'})
        self.assertRedirects(r, '/')

    def test_confirm_account_invalid_form(self):
        token1 = create_token(16)
        r = self.client.post("/%s/+new_account" % token1)
        self.assertTemplateUsed(r, 'registration/new_account.html')
        self.assertFormError(r, 'form', 'displayname', 'Required field.')

    def test_forgot_password_when_captcha_verification_fails(self):
        r = self.request_when_captcha_fails('/+forgot_password',
                                            {'email': 'mark@example.com'})
        self.assertTemplateUsed(r, 'registration/forgot_password.html')

    def test_forgot_password_email_when_account_active(self):
        self.client.post('/+forgot_password',
                         {'email': 'test@canonical.com'})

        self.assertEqual(len(mail.outbox), 1)
        mail.outbox = []

    def test_forgot_password_email_when_account_deactivated(self):
        """ Deactivated accounts can be reactivated using the reset password
        functionality. Bug #556878 """
        account = Account.objects.get_by_email('test@canonical.com')
        # make sure the account is deactivated
        account.status = AccountStatus.DEACTIVATED
        account.save()

        self.client.post('/+forgot_password',
                             {'email': 'test@canonical.com'})

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(account.preferredemail.email, 'test@canonical.com')

    def test_forgot_password_email_when_account_noaccount(self):
        account = Account.objects.get_by_email('test@canonical.com')
        # make sure the account is deactivated
        account.status = AccountStatus.NOACCOUNT
        account.save()

        self.client.post('/+forgot_password',
                             {'email': 'test@canonical.com'})

        self.assertEqual(len(mail.outbox), 1)
        mail.outbox = []
        self.assertEqual(account.preferredemail.email, 'test@canonical.com')

    def test_forgot_password_email_when_account_suspended(self):
        account = Account.objects.get_by_email('test@canonical.com')
        # make sure the account is suspended
        account.status = AccountStatus.SUSPENDED
        account.save()

        self.client.post('/+forgot_password',
                     {'email': 'test@canonical.com'})

        self.assertEqual(len(mail.outbox), 0)


    def test_forgot_password_invalid_form(self):
        r = self.client.post('/+forgot_password')
        self.assertEqual(r.status_code, 200)
        self.assertTemplateUsed(r, 'registration/forgot_password.html')
        self.assertFormError(r, 'form', 'email', 'Required field.')

    def test_forgot_password_with_token(self):
        token1 = create_token(16)
        self.client.post("/%s/+forgot_password" % token1,
                             {'email': 'test@canonical.com'})

        auth_token = self._session_token()
        self.assertEqual(auth_token.redirection_url, "/%s/+decide" % token1)

    def test_reset_password_when_account_active(self):
        r = self.client.post('/+forgot_password',
                             {'email': 'test@canonical.com'})

        account = Account.objects.get_by_email('test@canonical.com')
        # make sure the account is active
        account.status = AccountStatus.ACTIVE
        account.save()

        # confirm account
        data = {'password': 'Password1', 'passwordconfirm': 'Password1'}
        r = self.client.post(self._token_url('+resetpassword'), data)
        self.assertRedirects(r, '/')

    def test_reset_password_when_account_active_no_password(self):
        r = self.client.post('/+forgot_password',
                             {'email': 'test@canonical.com'})

        account = Account.objects.get_by_email('test@canonical.com')
        account.accountpassword.delete()
        # make sure the account is active
        account.status = AccountStatus.ACTIVE
        account.save()

        # confirm account
        data = {'password': 'Password1', 'passwordconfirm': 'Password1'}
        r = self.client.post(self._token_url('+resetpassword'), data)
        self.assertRedirects(r, '/')

    def test_reset_password_when_account_deactivated(self):
        """ Deactivated accounts can be reactivated using the reset password
        functionality. Bug #556878 """
        r = self.client.post('/+forgot_password',
                             {'email': 'test@canonical.com'})

        account = Account.objects.get_by_email('test@canonical.com')
        # make sure the account is deactivated
        account.status = AccountStatus.DEACTIVATED
        account.save()

        # confirm account
        data = {'password': 'Password1', 'passwordconfirm': 'Password1'}
        r = self.client.post(self._token_url('+resetpassword'), data)
        self.assertRedirects(r, '/')

    def test_reset_password_when_account_deactivated_no_preferred_email(self):
        account = Account.objects.get_by_email('test@canonical.com')
        # make sure the account is deactivated and preferred email is removed
        _preferred_email = account.preferredemail
        account.status = AccountStatus.DEACTIVATED
        account.save()
        for email_obj in account.emailaddress_set.all():
            email_obj.status = EmailStatus.NEW
            email_obj.save()
        self.assertEqual(account.preferredemail, None)

        r = self.client.post('/+forgot_password',
                             {'email': 'test@canonical.com'})

        self.assertEqual(len(mail.outbox), 1)
        mail.outbox = []

        # reset password
        data = {'password': 'Password1', 'passwordconfirm': 'Password1'}
        r = self.client.post(self._token_url('+resetpassword'), data)
        self.assertRedirects(r, '/')

        self.assertEqual(account.preferredemail.email, 'test@canonical.com')

        for email_obj in account.emailaddress_set.all():
            email_obj.status = EmailStatus.VALIDATED
            email_obj.save()
        account.preferredemail = _preferred_email

    def test_reset_password_when_account_noaccount(self):
        r = self.client.post('/+forgot_password',
                             {'email': 'test@canonical.com'})

        account = Account.objects.get_by_email('test@canonical.com')
        # make sure the account is deactivated
        account.status = AccountStatus.NOACCOUNT
        account.save()

        # confirm account
        data = {'password': 'Password1', 'passwordconfirm': 'Password1'}
        r = self.client.post(self._token_url('+resetpassword'), data)
        self.assertRedirects(r, '/')

    def test_reset_password_when_account_suspended(self):
        r = self.client.post('/+forgot_password',
                             {'email': 'test@canonical.com'})

        account = Account.objects.get_by_email('test@canonical.com')
        account.status = AccountStatus.SUSPENDED
        account.save()

        data = {'password': 'Password1', 'passwordconfirm': 'Password1'}

        # confirm account
        r = self.client.post(self._token_url('+resetpassword'), data)
        self.assertRedirects(r, '/+bad-token')

    def test_reset_password_invalid_form(self):
        # get valid session
        r = self.client.post('/+forgot_password',
                             {'email': 'test@canonical.com'})

        # test view
        r = self.client.post(self._token_url('+resetpassword'))
        self.assertTemplateUsed(r, 'registration/reset_password.html')
        self.assertFormError(r, 'form', 'password', 'Required field.')

    def test_reset_password_get(self):
        # get valid session
        r = self.client.post('/+forgot_password',
                             {'email': 'test@canonical.com'})

        # test view
        r = self.client.get(self._token_url('+resetpassword'))
        self.assertTemplateUsed(r, 'registration/reset_password.html')
        for context in r.context:
            self.assertEqual(context['form'].errors, {})

    def request_when_captcha_fails(self, url, data):
        class MockCaptcha(object):
            def __init__(self, *args):
                pass

            def verify(self, solution, ip_addr):
                self.message = 'no-challenge'
                return False

            @classmethod
            def new(cls, env):
                return cls()

        try:
            captcha_class = ui.Captcha
            ui.Captcha = MockCaptcha
            r = self.client.post(url, data)
        finally:
            ui.Captcha = captcha_class
        return r

    def test_new_account_when_form_validation_fails(self):
        r = self.client.post('/+new_account', {'email': 'test'})
        self.assertTemplateUsed(r, 'registration/new_account.html')

    def test_new_account_when_captcha_fails(self):
        r = self.request_when_captcha_fails('/+new_account',
                                            {'email': 'mark@example.com'})
        self.assertTemplateUsed(r, 'registration/new_account.html')

    def test_new_account_when_inactive(self):
        account = Account.objects.get_by_email('mark@example.com')

        for status, _ in AccountStatus._get_choices():
            account.status = status
            account.save()

            r = _post_new_account(self.client, email='mark@example.com')
            self.assertRedirects(r, '/+email-sent')

            if status == AccountStatus.ACTIVE:
                mails_sent = 1
            else:
                mails_sent = 0

            self.assertEqual(len(mail.outbox), mails_sent)
            mail.outbox = []

    @patch_object(logging.Logger, 'debug')
    def test_new_account_when_person_exists_and_account_not(self, mock_debug):
        person = Person.objects.create(displayname='Person')
        EmailAddress.objects.create(email='person@example.com',
                                    lp_person=person,
                                    status=EmailStatus.VALIDATED)

        _post_new_account(self.client)
        message = mock_debug.call_args[0][0]
        self.assertTrue('account for person' in message)

    def test_new_account_when_account_not_exists_no_token(self):
        _post_new_account(self.client)
        token_obj = self._session_token()
        self.assertEqual(token_obj.redirection_url, '/')

    def test_edit_account_template(self):
        r = self.client.post('/+login', {'email': 'mark@example.com',
                                         'password': 'test',
                                         'next': '/+edit'})
        r = self.client.get('/+edit')
        self.assertTemplateUsed(r, 'account/edit.html')

        content = r.content
        newpassword_pattern = '<input (.*)name="password"(.*)>'
        newpasswdconfirm_pattern = '<input (.*)name="passwordconfirm"(.*)>'
        password_pattern = re.compile('.*type="password".*')
        autocomplete_ptn = re.compile('.*class=".*disableAutoComplete.*".*')
        newpassword = re.search(newpassword_pattern, content)
        newpasswordconfirm = re.search(newpasswdconfirm_pattern, content)

        def is_password_type(container):
            data = container.group(0) + container.group(1)
            result = re.search(password_pattern, data) is not None
            return result

        def is_autocomplete(container):
            data = container.group(0) + container.group(1)
            result = re.search(autocomplete_ptn, data) is not None
            return result

        self.assertTrue(is_password_type(newpassword))
        self.assertTrue(is_password_type(newpasswordconfirm))
        self.assertTrue(is_autocomplete(newpassword))
        self.assertTrue(is_autocomplete(newpasswordconfirm))

    def _setup_account_with_new_email(self):
        account = Account.objects.get_by_email('mark@example.com')
        # create new email account
        email = account.emailaddress_set.create(
            email='newemail@example.com', status=EmailStatus.NEW)
        # create verification token
        token = self.create_token(at.LoginTokenType.VALIDATEEMAIL, email.email)
        return account, email, token

    def test_confirm_email_not_existing(self):
        self.authenticate()
        account, email, token = self._setup_account_with_new_email()
        # delete email account
        email.delete()
        # use verification token
        r = self.client.get("/token/%s/+newemail" % token.token)
        self.assertEquals(r.status_code, 404)

    def test_confirm_email_get(self):
        self.authenticate()

        account, email, token = self._setup_account_with_new_email()
        # use verification token
        self.client.get("/token/%s/+newemail" % token.token)

        # verify token has not been consumed, and email has not been
        # verified yet
        token = AuthToken.objects.get(token=token.token,
            token_type=LoginTokenType.VALIDATEEMAIL)
        self.assertEqual(token.date_consumed, None)
        email = EmailAddress.objects.get(email=email.email)
        self.assertEqual(email.status, EmailStatus.NEW)

    def test_confirm_email_as_another_user_fails(self):
        self.authenticate()
        account = Account.objects.get_by_email('test@canonical.com')

        r, token = self.get_email_token_validation_response(account)

        token_id = token.pk

        self.assertEquals(r.status_code, 404)
        self.assertEquals(0, at.AuthToken.objects.filter(pk=token_id).count())
        email = EmailAddress.objects.get(email='newemail2@example.com')
        self.assertEquals(EmailStatus.NEW, email.status)

    def test_config_email_when_not_logged_in_redirects_to_login(self):
        account = Account.objects.get_by_email('mark@example.com')

        r, token = self.get_email_token_validation_response(account)

        url = ('/+login?next=/token/%s/%%2Bnewemail/%s' %
               (token.token, 'newemail2%40example.com'))
        self.assertRedirects(r, url)

    def get_email_token_validation_response(self, account):
        email = account.emailaddress_set.create(
            email='newemail2@example.com', status=EmailStatus.NEW)
        token = self.create_token(at.LoginTokenType.VALIDATEEMAIL, email.email)
        url = ('/token/%s/+newemail/%s' %
               (token.token, 'newemail2%40example.com'))
        r = self.client.get(url)
        return (r, token)

    def test_suspended(self):
        r = self.client.get('/+suspended')
        self.assertTemplateUsed(r, 'account/suspended.html')

    def test_invalid_email_does_not_blow_up(self):
        r = _post_new_account(self.client, email='what~@ever.com')
        self.assertEquals(200, r.status_code)
        self.assertContains(r, 'Invalid email')

    def test_valid_email_redirects(self):
        r = _post_new_account(self.client, email='what@ever.com')
        self.assertRedirects(r, '/+email-sent')


class UIViewsLPModelTestCase(LPAccountTestCase):
    def test_new_account_when_person_does_not_exist(self):
        # create email address with broken foreign key
        email = EmailAddress.objects.get(email__iexact='mark@example.com')
        lp_person_id = email.lp_person_id
        email.lp_person_id = 0
        email.save()

        try:
            # test normal workflow
            r = _post_new_account(self.client, email='mark@example.com')
            self.assertRedirects(r, '/+email-sent')
        finally:
            # restore model integrity
            email.lp_person_id = lp_person_id
            email.save()


class CaptchaVerificationTestCase(BasicAccountTestCase):

    def setUp(self):
        self.client = Client(REMOTE_ADDR='127.0.0.1')
        mock_handler = MockHandler()
        mock_handler.set_next_response('false\nno-challenge')
        Captcha.opener = urllib2.build_opener(mock_handler)
        self.old_disable = settings.DISABLE_CAPTCHA_VERIFICATION
        settings.DISABLE_CAPTCHA_VERIFICATION = False

    def tearDown(self):
        settings.DISABLE_CAPTCHA_VERIFICATION = self.old_disable
        Captcha.opener = None

    def test_new_account_when_form_validation_fails(self):
        r = _post_new_account(self.client)
        self.assertTemplateUsed(r, 'registration/new_account.html')
        msg = 'It appears that our captcha service was unable to load'
        self.assertContains(r, msg)


class LanguagesTestCase(BasicAccountTestCase):

    def test_preffered_lang_is_preserved_after_logout(self):
        account = Account.objects.get_by_email("mark@example.com")
        account.preferredlanguage = 'es'
        account.save()

        self.client.login(username='mark@example.com', password='test')
        r = self.client.get('/')
        self.assertContains(r, 'es.png')

        self.client.get('/+logout')
        r = self.client.get('/')
        self.assertContains(r, 'es.png')

    def test_language_selected_before_login_is_preserved(self):
        self.disable_csrf()

        self.assertNotContains(self.client.get('/'), 'es.png')
        self.client.post('/set_language', {'language': 'es'})
        self.assertContains(self.client.get('/'), 'es.png')
        self.client.login(username='mark@example.com', password='test')
        self.assertContains(self.client.get('/'), 'es.png')

        self.reset_csrf()


class CookiesTestCase(BasicAccountTestCase):

    def setUp(self):
        super(CookiesTestCase, self).setUp()
        self._disable_cookie_check = decorators.disable_cookie_check
        decorators.disable_cookie_check = False

    def tearDown(self):
        decorators.disable_cookie_check = self._disable_cookie_check
        super(CookiesTestCase, self).tearDown()

    def _get(self, url, *params):
        r = self.client.get(url)
        # The following line makes this cookieless:
        self.client.cookies.clear()
        if 300 <= r.status_code < 400:
            location = r['Location']
            scheme, netloc, path, query, fragment = urlsplit(location)
            return self._get(path, QueryDict(query))
        else:
            return r

    def _check_url(self, url):
        r = self._get(url)
        self.assertContains(r, '<title>Cookies required</title>')

    def test_login(self):
        self._check_url('/+login')

    def test_forgot_password(self):
        self._check_url('/+forgot_password')

    def test_new_account(self):
        self._check_url('/+new_account')
