# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import logging
import urllib
import urllib2
import sys
from cStringIO import StringIO
from types import MethodType
from unittest import TestSuite, TextTestRunner, TestLoader

from django.conf import settings
from django.db import connection
from django.db.models import get_app, get_apps
from django.http import HttpRequest
from django.test import TestCase, TransactionTestCase
from django.test.simple import (build_suite, teardown_test_environment,
                                setup_test_environment)
from openid.consumer.discover import (
    OpenIDServiceEndpoint, OPENID_1_0_TYPE, OPENID_1_1_TYPE,
    OPENID_2_0_TYPE, OPENID_IDP_2_0_TYPE)
from openid.message import IDENTIFIER_SELECT

from identityprovider import decorators


def create_pgsql_functions(functions):
    cursor = connection.cursor()
    cursor.execute(PGSQL_FUNCTIONS['__init__'])
    for function in functions:
        cursor.execute(PGSQL_FUNCTIONS[function])
    cursor.execute("COMMIT")


class SQLCachedTestCase(TransactionTestCase):
    def _pre_setup(self):
        super(SQLCachedTestCase, self)._pre_setup()
        functions = getattr(self, 'pgsql_functions', [])
        create_pgsql_functions(functions)


class BasicAccountTestCase(SQLCachedTestCase):
    """ This SQLCachedTestCase subclass has some initial fixtures
    and functions set for basic tests. """
    fixtures = ['test']
    pgsql_functions = ['generate_openid_identifier']

    def reset_csrf(self):
        settings.MIDDLEWARE_CLASSES = self._MIDDLEWARE_CLASSES

    def disable_csrf(self):
        self._MIDDLEWARE_CLASSES = settings.MIDDLEWARE_CLASSES
        settings.MIDDLEWARE_CLASSES = (x for x in self._MIDDLEWARE_CLASSES
                                         if not 'csrf' in x.lower())


class LPAccountTestCase(BasicAccountTestCase):

    def setUp(self):
        super(LPAccountTestCase, self).setUp()

        # disable person constraint
        cursor = connection.cursor()
        # Committing any transactions, as you can't alter table inside one
        cursor.execute("COMMIT")
        cursor.execute('ALTER TABLE emailaddress DROP CONSTRAINT '
                       'emailaddress_person_fkey')
        cursor.execute("BEGIN")


    def tearDown(self):
        # enable person constraint
        cursor = connection.cursor()
        cursor.execute("COMMIT")
        cursor.execute('ALTER TABLE emailaddress ADD CONSTRAINT '
                       'emailaddress_person_fkey FOREIGN KEY (person) '
                       'REFERENCES lp_person(id) DEFERRABLE INITIALLY '
                       'DEFERRED')

        super(LPAccountTestCase, self).tearDown()


class AuthenticatedTestCase(BasicAccountTestCase):

    def setUp(self, disableCSRF=False):
        # We allow classes that inherit this one to disable CSRF when needed,
        # as CSRF breaks Django's test client:
        # http://code.djangoproject.com/ticket/9172
        if disableCSRF:
            self.old_middlewares = settings.MIDDLEWARE_CLASSES
            settings.MIDDLEWARE_CLASSES = tuple(x for x in self.old_middlewares
                                                  if not 'csrf' in x.lower())
        else:
            self.old_middlewares = settings.MIDDLEWARE_CLASSES
        self.login_email = "test@canonical.com"
        self.login_password = "test"

        self.client.login(username=self.login_email,
                          password=self.login_password)

    def tearDown(self):
        settings.MIDDLEWARE_CLASSES = self.old_middlewares


class OpenIDProviderTestCase(SQLCachedTestCase):
    pgsql_functions = ['generate_openid_identifier']

    def get_query(self, response):
        query_start = response['Location'].find('?')
        query_str = response['Location'][query_start+1:]
        query_items = map(tuple,
            [item.split('=') for item in query_str.split('&')])
        query = dict(query_items)
        return query




def make_identifier_select_endpoint(protocol_uri):
    """Create an endpoint for use in OpenID identifier select mode.

    :arg protocol_uri: The URI for the OpenID protocol version.  This
        should be one of the OPENID_X_Y_TYPE constants.

    If the OpenID 1.x protocol is selected, the endpoint will be
    suitable for use with Launchpad's non-standard identifier select
    workflow.
    """
    assert protocol_uri in [
        OPENID_1_0_TYPE, OPENID_1_1_TYPE, OPENID_2_0_TYPE], (
        "Unexpected protocol URI: %s" % protocol_uri)

    endpoint = OpenIDServiceEndpoint()
    endpoint.server_url = settings.SSO_PROVIDER_URL
    if protocol_uri == OPENID_2_0_TYPE:
        endpoint.type_uris = [OPENID_IDP_2_0_TYPE]
    else:
        endpoint.type_uris = [protocol_uri]
        endpoint.claimed_id = IDENTIFIER_SELECT
        endpoint.local_id = IDENTIFIER_SELECT
    return endpoint


class MockRequest(HttpRequest):

    def __init__(self, path):
        super(HttpRequest, self).__init__()
        self.path = path

        class MockSession(dict):
            def flush(self):
                pass

        self.session = MockSession()

    def get_full_path(self):
        return self.path


# This function was taken from Michael Foord's python mock object
# Please see the license file in the thirdparty/mock directory.
def _dot_lookup(thing, comp, import_path):
    try:
        return getattr(thing, comp)
    except AttributeError:
        __import__(import_path)
        return getattr(thing, comp)


# This function was taken from Michael Foord's python mock object
# Please see the license file in the thirdparty/mock directory.
def _importer(target):
    components = target.split('.')
    import_path = components.pop(0)
    thing = __import__(import_path)

    for comp in components:
        import_path += ".%s" % comp
        thing = _dot_lookup(thing, comp, import_path)
    return thing


def build_test(label):
    test = _importer(label)
    if type(test) is MethodType:
        TestClass = test.im_class
        method_name = test.im_func.__name__
        return TestClass(method_name)
    elif test is TestCase:
        return TestLoader().loadTestsFromTestCase(test)
    else:
        # test must be module
        return TestLoader().loadTestsFromModule(test)


class NullDevice(object):
    "A class that simulates writing to /dev/null."
    def write(self, s):
        pass
    def flush(self):
        pass


# This function was adapted from the django project.
# Please see the license file in the thirdparty/django directory.
def run_tests(test_labels, verbosity=1, interactive=True, extra_tests=[]):
    # disable stderr while running the tests
    old_stderr = sys.stderr
    #sys.stderr = NullDevice()

    setup_test_environment()
    decorators.disable_cookie_check = True
    settings.DEBUG = False
    suite = TestSuite()

    if test_labels:
        for label in test_labels:
            if '.' in label:
                suite.addTest(build_test(label))
            else:
                app = get_app(label)
                suite.addTest(build_suite(app))
    else:
        for app in get_apps():
            suite.addTest(build_suite(app))

    if extra_tests:
        for test in extra_tests:
            suite.addTest(test)

    old_name = settings.DATABASE_NAME
    from django.db import connection
    connection.creation.create_test_db(verbosity, autoclobber=not interactive)

    # disable logging
    logging.disable(logging.CRITICAL)

    result = TextTestRunner(verbosity=verbosity).run(suite)
    settings.DATABASE_NAME = "test_{0}".format(settings.DATABASE_NAME)
    connection.creation.destroy_test_db(old_name, verbosity)
    teardown_test_environment()

    # re-enable stderr
    sys.stderr = old_stderr
    return len(result.failures) + len(result.errors)


PGSQL_FUNCTIONS = {
    '__init__': '''
DROP LANGUAGE IF EXISTS plpythonu CASCADE;
CREATE LANGUAGE plpythonu;
''',
    'generate_openid_identifier': '''
CREATE OR REPLACE FUNCTION generate_openid_identifier()
  RETURNS text AS
$BODY$
    from random import choice

    # Non display confusing characters.
    chars = '34678bcdefhkmnprstwxyzABCDEFGHJKLMNPQRTWXY'

    # Character length of tokens. Can be increased, decreased or even made
    # random - Launchpad does not care. 7 means it takes 40 bytes to store
    # a null-terminated Launchpad identity URL on the current domain name.
    length=7

    loop_count = 0
    while loop_count < 20000:
        # Generate a random openid_identifier
        oid = ''.join(choice(chars) for count in range(length))

        # Check if the oid is already in the db, although this is pretty
        # unlikely
        rv = plpy.execute("""
            SELECT COUNT(*) AS num FROM Account WHERE openid_identifier = '%%s'
            """ %% oid, 1)
        if rv[0]['num'] == 0:
            return oid
        loop_count += 1
        if loop_count == 1:
            plpy.warning(
                'Clash generating unique openid_identifier. '
                'Increase length if you see this warning too much.')
    plpy.error(
        "Unable to generate unique openid_identifier. "
        "Need to increase length of tokens.")
$BODY$
  LANGUAGE 'plpythonu' VOLATILE
  COST 100;
''',
}


class MockHandler(urllib2.HTTPHandler):

    def set_next_response(self, msg):
        self.next_response = msg

    def http_open(self, req):
        response = StringIO(self.next_response)
        resp = urllib.addinfourl(response, {}, req.get_full_url())
        resp.code = 200
        resp.msg = 'OK'
        return resp


def add_user_to_team(account, teamname):
    """ Note: will not work on the staging or production databases as those are
        replicated from LP and read-only.
    """
    from datetime import date
    from identityprovider.models.person import Person
    from identityprovider.models.account import (Account, LPOpenIdIdentifier)
    from identityprovider.models.team import (TeamMembership, TeamParticipation)

    memberships = TeamMembership.get_team_memberships_for_user([teamname],
                                                               account, True)
    if len(memberships) > 0:
        return

    p = account.person
    if p is None:
        p = Person.objects.create(lp_account=account.id,
                                  displayname=account.displayname,
                                  name=get_unique_username())

    try:
        oi = LPOpenIdIdentifier.objects.get(lp_account=account.id)
    except LPOpenIdIdentifier.DoesNotExist:
        oi = LPOpenIdIdentifier.objects.create(
                identifier=account.openid_identifier, lp_account=account.id)

    try:
        t = Person.objects.get(name=teamname)
    except Person.DoesNotExist:
        t = Person.objects.create(displayname="Team %s" % teamname, teamowner=p,
                                  name=teamname)

    now = date.today()
    exp = date(day=now.day, month=now.month, year=now.year+1)
    TeamMembership.objects.create(person=p, date_expires=exp, team=t, status=1)
    TeamParticipation.objects.create(team=t, person=p)

def get_unique_username():
    from identityprovider.models.person import Person
    i = 0
    while True:
        try:
            username = "user%d" % i
            i += 1
            Person.objects.get(name=username)
        except Person.DoesNotExist:
            break
    return username

