# Copyright 2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import

import os.path
import shutil
import sys
import textwrap

from fabric.api import (
    env,
    local,
)
from fabric.context_managers import cd

from .django import *
from .postgresql import *
from .upgrade_from_package import *
from .virtualenv import *

# make sure the virtualenv is automatically activated
setup_virtualenv()


# constants
# =========

# bootstrap
BUILD_DEPENDENCIES = ['python-profiler']
PSYCOPG2_CONFLICTS = ['python-egenix-mx-base-dev']

# translations
APPS = ["identityprovider"]
TRANSLATIONS_BRANCH = 'lp:~canonical-isd-hackers/canonical-identity-provider/translations'


# fab targets
# ===========

# environment
def bootstrap(hudson=False):
    """Bootstrap the development environment"""
    env.hudson = hudson

    # run sanity checks
    _check_bootstrap_dependencies()
    _check_psycopg2_conflicts()
    # bootstrap
    setup_virtualenv()
    install_dependencies()
    _setup_configuration()


def clean():
    """Clean up compiled and backup files."""
    clean_coverage()
    local("find . -name '*.~*' -delete")
    local("find . -name '*.pyc' -delete")


def clean_coverage():
    """Remove coverage report files."""
    with cd('django_project'):
        local("rm -rf .coverage coverage.d coverage.xml")


# development
def test(functional=False, coverage=False):
    """Run the test suite."""
    hudson = env.get('hudson', False)

    cmd = ['cd django_project &&',
           'PYTHONPATH=. nosetests --with-django identityprovider']
    if not functional:
        cmd.append("--eval-attr='not functional'")
    if coverage and not hudson:
        cmd.append('--with-coverage --cover-package=identityprovider')
    if hudson:
        # add support for cobertura
        cmd.append('--with-xcoverage --cover-package=identityprovider')
        cmd.append('--with-xunit')
    virtualenv(' '.join(cmd), capture=False)


def hudson():
    """Run the tests for hudson."""
    bootstrap(hudson=True)
    resetdb()
    # the functional tests depend on some data found in the fixtures
    virtualenv('python django_project/manage.py loaddata test')
    test(functional=True, coverage=True)


def run(*args):
    """Run SSO using devserver."""
    virtualenv('python django_project/manage.py runserver ' + ' '.join(args),
               capture=False)

# deploy
def makemessages():
    """Create/Update translation strings in .po files."""
    django_settings = get_django_settings('SUPPORTED_LANGUAGES')
    supported_languages = django_settings['SUPPORTED_LANGUAGES']
    for app in APPS:
        for lang in supported_languages:
            with cd(app):
                cmd = ("python ../django_project/manage.py makemessages "
                    "-l {0} -e .html,.txt")
                if '-' in lang:
                    language, country = lang.split('-')
                    locale_name = "{0}_{1}".format(language, country.upper())
                else:
                    locale_name = lang
                virtualenv(cmd.format(locale_name), capture=False)


def compilemessages(args=''):
    """Compile .po translation files into binary (.mo)."""
    cmd = 'python ../django_project/manage.py compilemessages'
    if args:
        cmd += " %s" % args
    for app in APPS:
        with cd(app):
            virtualenv(cmd, capture=False)


def deploy(config_branch):
    """Deploy latest code and translations."""
    build_and_install_deb(config_branch)
    deploy_translations()


def deploy_translations():
    """Update and compile translations and deploy to server."""
    tmp_dir = os.tempnam()
    local("bzr branch %s %s" % (TRANSLATIONS_BRANCH, tmp_dir))
    try:
        args =_setup_locale_basedir(tmp_dir)
        compilemessages(args)
        _submit_translations(tmp_dir)
        update_translations(TRANSLATIONS_BRANCH)
    finally:
        shutil.rmtree(tmp_dir)


# helpers
# =======
def _check_bootstrap_dependencies():
    """Check dependencies required for bootstrap."""
    required = []
    cmd = "dpkg -l %s 2> /dev/null | grep '^ii' | wc -l"
    for pkg in BUILD_DEPENDENCIES:
        output = local(cmd % pkg).strip()
        if output != '1':
            required.append(pkg)
    if required:
        print "Please install the following packages, as they are required"
        print "in order to build some of the dependencies:"
        for pkg in required:
            print pkg
        sys.exit(1)


def _check_psycopg2_conflicts():
    """Check for libraries conflicting with psycopg2."""
    conflicting = []
    cmd = "dpkg -l %s 2> /dev/null | grep '^ii' | wc -l"
    for pkg in PSYCOPG2_CONFLICTS:
        output = local(cmd % pkg).strip()
        if output != '0':
            conflicting.append(pkg)
    if conflicting:
        print "Please remove the following packages, as their presence will"
        print "produce the psycopg2 library to be built incorrectly:"
        for pkg in conflicting:
            print pkg
        sys.exit(1)


def _setup_configuration():
    """Setup the configuration."""
    if not os.path.exists('django_project/local.cfg'):
        _create_local_cfg()


def _create_local_cfg():
    if env.get('hudson', False):
        db_host = ''
    else:
        db_host = "%s/db" % env.virtualenv

    config = textwrap.dedent("""
        [__noschema__]
        basedir = %s
        db_host = %s

        [__main__]
        includes = config/devel-13.cfg
    """ % (os.path.abspath(os.curdir), db_host))

    if db_host:
        config += "test_dsn = host=%(db_host)s dbname=%(db_name)s user=%(db_user)s"

    with file('django_project/local.cfg', 'w') as local_cfg:
        local_cfg.write(config)


def _setup_locale_basedir(translations_path):
    """Prepare the locale_paths."""
    locale_paths = '\n'.join(["%s/locale/%s" % (translations_path, app)
        for app in APPS])
    args = "--django_locale_paths=\"%s\"" % locale_paths
    return args


def _submit_translations(translations_path):
    """Push updated stable translations to upstream branch."""
    with cd(translations_path):
        output = local("bzr status")
        if output:
            output = local("bzr revno %s" % TRANSLATIONS_BRANCH)
            revno = int(output.strip())
            commit_message = ("compiled latest translations (%s@%d)" %
                (TRANSLATIONS_BRANCH, revno))
            local("bzr commit -m '%s'" % commit_message)
            local("bzr push %s" % TRANSLATIONS_BRANCH)
        else:
            print 'Nothing to do; translations are up to date.'


def manage(command, *args):
    """Run manage.py command"""
    virtualenv("python django_project/manage.py {0} {1}".format(
        command, " ".join(args)), capture=False,)
