#   -*- coding: utf-8 -*-

"""Fabric commands for building a package for a direct server upgrade.

The local machine must have bzr-builder and pbuilder, and a lucid chroot
setup as the default pbuilder distribution:

    $ sudo pbuilder create --distribution lucid

"""

from __future__ import absolute_import
from __future__ import with_statement

__metaclass__ = type
__all__ = [
    'build_and_install_deb',
    'build_binary_packages',
    'build_source_package',
    'clean_local_branch',
    'install_binary_packages',
    'migrate_database',
    'preflight_check',
    'restart_services',
    'update_configs',
    'update_translations',
    ]

import os
import shutil
import tempfile
from datetime import datetime
from glob import glob
from textwrap import dedent

from fabric.api import (
    env,
    local,
    put,
    run,
    sudo,
)
from fabric.contrib.files import exists


def clean_local_branch():
    local("find . -name '*.pyc' -delete")


def build_source_package(build_dir, recipe_filename='sso.recipe',
    distroseries='lucid'):
    clean_local_branch()
    # We use a recipe to ensure that we always generate later versions
    # with a timestamp. The recipe refers to the local branch '.' rather
    # than pulling from the network.
    recipe = dedent("""\
        # bzr-builder format 0.2 deb-version {debupstream}-0~{revno}+{time}
        .""")
    recipe_path = os.path.join(build_dir, recipe_filename)
    with open(recipe_path, 'w+') as recipe_file:
        recipe_file.write(recipe)

    local('bzr dailydeb --distribution={0} {1} {2}'.format(
        distroseries, recipe_path, build_dir))
    dsc_file = glob(os.path.join(build_dir, '*.dsc'))[0]

    print("Created source package at {0}.".format(dsc_file))
    return dsc_file


def build_binary_packages(build_dir, source_dsc):
    print("Building binary package for {0}".format(source_dsc))
    local('sudo pbuilder build --buildresult {0} {1}'.format(
        build_dir, source_dsc))
    deb_files = glob(os.path.join(build_dir, "*.deb"))
    print("Created binary at {0}.".format(deb_files))
    return deb_files


def migrate_database(config_dir=None):
    if config_dir is None:
        # By default, the sso config directory is assumed to be
        # /home/username/django_project.
        config_dir = 'django_project'
    run(config_dir + '/manage.py syncdb')


def preflight_check(config_dir='django_project'):
    run('COLUMNS=40 %s/manage.py preflight ' % config_dir)


def restart_services():
    sudo('service apache2 graceful', shell=False)


def install_binary_packages(*deb_files):
    files = []
    for deb_file in deb_files:
        put(deb_file, '.')
        deb_filename = os.path.basename(deb_file)
        files.append(deb_filename)
    sudo('dpkg -i {0}'.format(' '.join(files)), shell=False)


LOCAL_CONFIG = """
[__noschema__]
me = %s

[__main__]
includes = config/staging.cfg
           config/uec_local.cfg
"""


def update_configs(config_branch, config_dir='django_project'):
    """Backup current config files and pull in the latest from a bzr branch.

    If it exists, local.cfg is left unmodified.
    """
    timestamp = datetime.utcnow().isoformat()
    backup_dir = "%s.backup.%s" % (config_dir, timestamp)
    if exists(config_dir):
        run("mv %s %s" % (config_dir, backup_dir))
    put_bzr_branch(config_branch, config_dir)
    if exists(backup_dir + '/local.cfg'):
        run("cp %s/local.cfg %s" % (backup_dir, config_dir))
    else:
        # Create a local.cfg on the server from our template.
        local_file, local_path = tempfile.mkstemp()
        local_file.write(LOCAL_CONFIG % env.host_string)
        local_file.close()
        put(local_path, "%s/local.cfg" % config_dir)
        os.remove(local_path)


def update_translations(translations_branch, translations_dir='translations'):
    """Backup current translation files and pull in the latest from a bzr
    branch.
    """
    timestamp = datetime.utcnow().isoformat()
    backup_dir = "%s.backup.%s" % (translations_dir, timestamp)
    if exists(translations_dir):
        run("mv %s %s" % (translations_dir, backup_dir))
    put_bzr_branch(translations_branch, translations_dir)


def put_bzr_branch(url, name):
    """
    Copy bazaar branch from launchpad to remote host

    This have to be done with local machine as proxy, because remote machine
    cannot have access to private projects on launchpad. We don't want to put
    there our private ssh keys.

    :param url: bazaar url to the branch, it can start with lp:
    :type  url: str

    :param name: name of the directory in which branch will be located remotely
    :type  name: str

    """
    local("rm -rf /tmp/%s" % name)

    local("bzr branch %s /tmp/%s" % (url, name))
    local("cd /tmp && tar cjf %s.tar.bz2 %s" % (name, name))
    put("/tmp/%s.tar.bz2" % name, '~/')

    local("rm -rf /tmp/%s.tar.bz2 /tmp/%s" % (name, name))

    run("tar xjf %s.tar.bz2" % name)


def build_and_install_deb(config_branch, build_dir=None, distroseries='lucid'):
    delete_dir_after_build = False
    if build_dir is None:
        build_dir = tempfile.mkdtemp()
        delete_dir_after_build = True

    dsc_file = build_source_package(build_dir, distroseries=distroseries)
    binary_files = build_binary_packages(build_dir, dsc_file)
    install_binary_packages(*binary_files)
    migrate_database()
    update_configs(config_branch)
    preflight_check()
    restart_services()
    print("Package installed and services restarted.")

    if delete_dir_after_build:
        shutil.rmtree(build_dir, ignore_errors=True)
